@@ -34,19 +34,31 @@
 #include "peas-extension.h"
 #include "peas-dirs.h"
 #include "peas-debug.h"
+#include "peas-helpers.h"
 
 /**
  * SECTION:peas-engine
  * @short_description: Engine at the heart of the Peas plugin system.
  * @see_also: #PeasPluginInfo
  *
- * The #PeasEngine is the object which manages the plugins.  Its role is twofold:
- * First it will fetch all the information about the available plugins from all
- * the registered plugin directories, and second it will provide you an API to
- * load, control and unload the plugins from within your application.
+ * The #PeasEngine is the object which manages the plugins.
+ *
+ * Its role is twofold:
+ * <itemizedlist>
+ *   <listitem>
+ *     <para>it will fetch all the information about the available plugins
+ *     from all the registered plugin directories;</para>
+ *   </listitem>
+ *   <listitem>
+ *     <para>it will provide you an API to load, control and unload your
+ *     plugins and their extensions from within your application.</para>
+ *   </listitem>
+ * </itemizedlist>
  **/
 G_DEFINE_TYPE (PeasEngine, peas_engine, G_TYPE_OBJECT);
 
+static PeasEngine *default_engine = NULL;
+
 /* Signals */
 enum {
   LOAD_PLUGIN,
@@ -59,9 +71,8 @@ static guint signals[LAST_SIGNAL];
 /* Properties */
 enum {
   PROP_0,
-  PROP_APP_NAME,
-  PROP_BASE_MODULE_DIR,
-  PROP_SEARCH_PATHS
+  PROP_PLUGIN_LIST,
+  PROP_LOADED_PLUGINS
 };
 
 typedef struct _LoaderInfo LoaderInfo;
@@ -71,10 +82,13 @@ struct _LoaderInfo {
   PeasObjectModule *module;
 };
 
+typedef struct _SearchPath {
+  gchar *module_dir;
+  gchar *data_dir;
+} SearchPath;
+
 struct _PeasEnginePrivate {
-  gchar *app_name;
-  gchar *base_module_dir;
-  gchar **search_paths;
+  GList *search_paths;
 
   GList *plugin_list;
   GHashTable *loaders;
@@ -95,12 +109,14 @@ load_plugin_info (PeasEngine  *engine,
   const gchar *module_name;
 
   info = _peas_plugin_info_new (filename,
-                                engine->priv->app_name,
                                 module_dir,
                                 data_dir);
 
   if (info == NULL)
-    return;
+    {
+      g_warning ("Error loading '%s'", filename);
+      return;
+    }
 
   /* If a plugin with this name has already been loaded
    * drop this one (user plugins override system plugins) */
@@ -113,7 +129,6 @@ load_plugin_info (PeasEngine  *engine,
 
 static void
 load_dir_real (PeasEngine  *engine,
-               const gchar *extension,
                const gchar *module_dir,
                const gchar *data_dir,
                guint        recursions)
@@ -122,13 +137,13 @@ load_dir_real (PeasEngine  *engine,
   GDir *d;
   const gchar *dirent;
 
-  g_debug ("Loading %s/*%s...", module_dir, extension);
+  g_debug ("Loading %s/*.plugin...", module_dir);
 
   d = g_dir_open (module_dir, 0, &error);
 
   if (!d)
     {
-      g_message ("%s", error->message);
+      g_debug ("%s", error->message);
       g_error_free (error);
       return;
     }
@@ -137,11 +152,11 @@ load_dir_real (PeasEngine  *engine,
     {
       gchar *filename = g_build_filename (module_dir, dirent, NULL);
 
-      if (g_str_has_suffix (dirent, extension))
+      if (g_str_has_suffix (dirent, ".plugin"))
         load_plugin_info (engine, filename, module_dir, data_dir);
 
       else if (recursions > 0 && g_file_test (filename, G_FILE_TEST_IS_DIR))
-        load_dir_real (engine, extension, filename, data_dir, recursions - 1);
+        load_dir_real (engine, filename, data_dir, recursions - 1);
 
       g_free (filename);
     }
@@ -162,29 +177,69 @@ load_dir_real (PeasEngine  *engine,
 void
 peas_engine_rescan_plugins (PeasEngine *engine)
 {
-  gchar **sp;
-  gchar *extension;
-  guint i;
+  GList *item;
 
   g_return_if_fail (PEAS_IS_ENGINE (engine));
 
-  sp = engine->priv->search_paths;
-  if (sp == NULL)
+  if (engine->priv->search_paths == NULL)
     {
       g_debug ("No search paths where provided");
       return;
     }
 
-  /* Compute the extension of the plugin files. */
-  extension = g_strdup_printf (".%s-plugin", engine->priv->app_name);
-  for (i = 0; extension[i] != '\0'; ++i)
-    extension[i] = g_ascii_tolower (extension[i]);
-
   /* Go and read everything from the provided search paths */
-  for (i = 0; sp[i] != NULL; i += 2)
-    load_dir_real (engine, extension, sp[i], sp[i + 1], 1);
+  for (item = engine->priv->search_paths; item != NULL; item = item->next)
+    {
+      SearchPath *sp = (SearchPath *) item->data;
+      load_dir_real (engine, sp->module_dir, sp->data_dir, 1);
+    }
 
-  g_free (extension);
+  g_object_notify (G_OBJECT (engine), "plugin-list");
+}
+
+/**
+ * peas_engine_add_search_path:
+ * @engine: A #PeasEngine.
+ * @module_dir: the plugin module directory.
+ * @data_dir: (allow-none): the plugin data directory.
+ *
+ * This function appends a search path to the list of paths where to
+ * look for plugins.
+ *
+ * A so-called "search path" actually consists of a couple of both a
+ * module directory (where the shared libraries or language modules
+ * lie) and a data directory (where the plugin data is).
+ *
+ * The #PeasPlugin will be able to use a correct data dir depending on
+ * where it is installed, hence allowing to keep the plugin agnostic
+ * when it comes to installation location: the same plugin can be
+ * installed either in the system path or in the user's home directory,
+ * without taking other special care than using
+ * peas_plugin_info_get_data_dir() when looking for its data files.
+ *
+ * If @data_dir is %NULL, then it is set to the same value as
+ * @module_dir.
+ */
+void
+peas_engine_add_search_path (PeasEngine *engine,
+                             const gchar *module_dir,
+                             const gchar *data_dir)
+{
+  SearchPath *sp;
+
+  g_return_if_fail (PEAS_IS_ENGINE (engine));
+  g_return_if_fail (module_dir != NULL);
+
+  sp = g_slice_new (SearchPath);
+  sp->module_dir = g_strdup (module_dir);
+  sp->data_dir = g_strdup (data_dir ? data_dir : module_dir);
+
+  /* Appending to a list is bad, but is easier to handle wrt refreshing
+   * the plugin list. */
+  engine->priv->search_paths = g_list_append (engine->priv->search_paths, sp);
+
+  load_dir_real (engine, sp->module_dir, sp->data_dir, 1);
+  g_object_notify (G_OBJECT (engine), "plugin-list");
 }
 
 static guint
@@ -238,6 +293,12 @@ add_loader (PeasEngine       *engine,
 static void
 peas_engine_init (PeasEngine *engine)
 {
+  /* Set the default engine pointer, for peas_engine_get_default().
+   * We only allow one single instance of a PeasEngine subclass. */
+  g_assert (default_engine == NULL);
+  default_engine = engine;
+  g_object_add_weak_pointer (G_OBJECT (engine), (gpointer *) &default_engine);
+
   if (!g_module_supported ())
     {
       g_warning ("libpeas is not able to initialize the plugins engine.");
@@ -296,14 +357,9 @@ peas_engine_set_property (GObject      *object,
 
   switch (prop_id)
     {
-    case PROP_APP_NAME:
-      engine->priv->app_name = g_value_dup_string (value);
-      break;
-    case PROP_BASE_MODULE_DIR:
-      engine->priv->base_module_dir = g_value_dup_string (value);
-      break;
-    case PROP_SEARCH_PATHS:
-      engine->priv->search_paths = g_value_dup_boxed (value);
+    case PROP_LOADED_PLUGINS:
+      peas_engine_set_loaded_plugins (engine,
+                                      (const gchar **) g_value_get_boxed (value));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -321,14 +377,13 @@ peas_engine_get_property (GObject    *object,
 
   switch (prop_id)
     {
-    case PROP_APP_NAME:
-      g_value_set_string (value, engine->priv->app_name);
+    case PROP_PLUGIN_LIST:
+      g_value_set_pointer (value,
+                           (gpointer) peas_engine_get_plugin_list (engine));
       break;
-    case PROP_BASE_MODULE_DIR:
-      g_value_set_string (value, engine->priv->base_module_dir);
-      break;
-    case PROP_SEARCH_PATHS:
-      g_value_set_boxed (value, engine->priv->search_paths);
+    case PROP_LOADED_PLUGINS:
+      g_value_take_boxed (value,
+                          (gconstpointer) peas_engine_get_loaded_plugins (engine));
       break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
@@ -358,10 +413,18 @@ peas_engine_finalize (GObject *object)
   for (item = engine->priv->plugin_list; item; item = item->next)
     _peas_plugin_info_unref (PEAS_PLUGIN_INFO (item->data));
 
+  /* free the search path list */
+  for (item = engine->priv->search_paths; item; item = item->next)
+    {
+      SearchPath *sp = (SearchPath *) item->data;
+
+      g_free (sp->module_dir);
+      g_free (sp->data_dir);
+      g_slice_free (SearchPath, sp);
+    }
+
   g_list_free (engine->priv->plugin_list);
-  g_strfreev (engine->priv->search_paths);
-  g_free (engine->priv->app_name);
-  g_free (engine->priv->base_module_dir);
+  g_list_free (engine->priv->search_paths);
 
   G_OBJECT_CLASS (peas_engine_parent_class)->finalize (object);
 }
@@ -381,84 +444,48 @@ peas_engine_class_init (PeasEngineClass *klass)
   klass->unload_plugin = peas_engine_unload_plugin_real;
 
   /**
-   * PeasEngine:app-name:
+   * PeasEngine:plugin-list:
    *
-   * The application name. Filename extension and section header for
-   * #PeasPluginInfo files are actually derived from this value.
+   * The list of found plugins.
    *
-   * For instance, if app-name is "Gedit", then info files will have
-   * the .gedit-plugin extension, and the engine will look for a
-   * "Gedit Plugin" section in it to load the plugin data.
+   * This will be modified when peas_engine_rescan_plugins() is called.
+   *
+   * Note that the list belongs to the engine and should not be modified
+   * or freed.
    */
   g_object_class_install_property (object_class,
-                                   PROP_APP_NAME,
-                                   g_param_spec_string ("app-name",
-                                                        "Application Name",
-                                                        "The application name",
-                                                        "Peas",
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
+                                   PROP_PLUGIN_LIST,
+                                   g_param_spec_pointer ("plugin-list",
+                                                         "Plugin list",
+                                                         "The list of found plugins",
+                                                         G_PARAM_READABLE |
+                                                         G_PARAM_STATIC_STRINGS));
 
   /**
-   * PeasEngine:base-module-dir:
+   * PeasEngine:loaded-plugins:
    *
-   * The base application directory for binding modules lookup.
+   * The list of loaded plugins.
    *
-   * Each loader module will load its modules from a subdirectory of
-   * the base module directory. For instance, the python loader will
-   * look for python modules in "${base-module-dir}/python/".
-   */
-  g_object_class_install_property (object_class,
-                                   PROP_BASE_MODULE_DIR,
-                                   g_param_spec_string ("base-module-dir",
-                                                        "Base module dir",
-                                                        "The base application dir for binding modules lookup",
-                                                        NULL,
-                                                        G_PARAM_READWRITE |
-                                                        G_PARAM_CONSTRUCT_ONLY |
-                                                        G_PARAM_STATIC_STRINGS));
-
-  /**
-   * PeasEngine:search-paths:
+   * This will be modified when peas_engine_load_plugin() or
+   * peas_engine_unload_plugin() is called.
    *
-   * The list of path where to look for plugins.
-   *
-   * A so-called "search path" actually consists on a couple of both a
-   * module directory (where the shared libraries or language modules
-   * lie) and a data directory (where the plugin data is).
-   *
-   * The #PeasPlugin will be able to use a correct data dir depending on
-   * where it is installed, hence allowing to keep the plugin agnostic
-   * when it comes to installation location: the same plugin can be
-   * installed both in the system path or in the user's home directory,
-   * without taking other special care than using
-   * egg_plugin_get_data_dir() when looking for its data files.
-   *
-   * Concretely, this property contains a NULL-terminated array of
-   * strings, whose even-indexed strings are module directories, and
-   * odd-indexed ones are the associated data directories.  Here is an
-   * example of such an array:
+   * This can be used with GSettings to save the loaded plugins by binding
+   * to this property after instantiating the engine by doing:
    * |[
-   * gchar const * const search_paths[] = {
-   *         // Plugins in ~/.config
-   *         g_build_filename (g_get_user_config_dir (), "example/plugins", NULL),
-   *         g_build_filename (g_get_user_config_dir (), "example/plugins", NULL),
-   *         // System plugins
-   *         EXAMPLE_PREFIX "/lib/example/plugins/",
-   *         EXAMPLE_PREFIX "/share/example/plugins/",
-   *         NULL
-   * };
+   *   g_settings_bind (gsettings_object,
+   *                    LOADED_PLUGINS_KEY,
+   *                    engine,
+   *                    "loaded-plugins",
+   *                    G_SETTINGS_BIND_DEFAULT);
    * ]|
    */
   g_object_class_install_property (object_class,
-                                   PROP_SEARCH_PATHS,
-                                   g_param_spec_boxed ("search-paths",
-                                                       "Search paths",
-                                                       "The list of paths where to look for plugins",
+                                   PROP_LOADED_PLUGINS,
+                                   g_param_spec_boxed ("loaded-plugins",
+                                                       "Loaded plugins",
+                                                       "The list of loaded plugins",
                                                        G_TYPE_STRV,
                                                        G_PARAM_READWRITE |
-                                                       G_PARAM_CONSTRUCT_ONLY |
                                                        G_PARAM_STATIC_STRINGS));
 
   /**
@@ -467,6 +494,12 @@ peas_engine_class_init (PeasEngineClass *klass)
    * @info: A #PeasPluginInfo.
    *
    * The load-plugin signal is emitted when a plugin is being loaded.
+   *
+   * The plugin is being loaded in the default handler. Hence, if you want to
+   * perform some action before the plugin is loaded, you should use
+   * g_signal_connect(), but if you want to perform some action *after* the
+   * plugin is loaded (the most common case), you should use
+   * g_signal_connect_after().
    */
   signals[LOAD_PLUGIN] =
     g_signal_new ("load-plugin",
@@ -486,6 +519,12 @@ peas_engine_class_init (PeasEngineClass *klass)
    * @info: A #PeasPluginInfo.
    *
    * The unload-plugin signal is emitted when a plugin is being unloaded.
+   *
+   * The plugin is being unloaded in the default handler. Hence, if you want
+   * to perform some action before the plugin is unloaded (the most common
+   * case), you should use g_signal_connect(), but if you want to perform some
+   * action after the plugin is unloaded (the most common case), you should
+   * use g_signal_connect_after().
    */
   signals[UNLOAD_PLUGIN] =
     g_signal_new ("unload-plugin",
@@ -505,65 +544,84 @@ peas_engine_class_init (PeasEngineClass *klass)
   peas_debug_init ();
 }
 
+static PeasObjectModule *
+try_to_open_loader_module (const gchar *loader_id,
+                           gboolean     in_subdir)
+{
+  gchar *tmp_dirname;
+  gchar *loader_dirname;
+  gchar *loader_basename;
+  PeasObjectModule *module;
+
+  if (in_subdir)
+    {
+      tmp_dirname = peas_dirs_get_plugin_loaders_dir ();
+      loader_dirname = g_build_filename (tmp_dirname, loader_id, NULL);
+      g_free (tmp_dirname);
+    }
+  else
+    {
+      loader_dirname = peas_dirs_get_plugin_loaders_dir ();
+    }
+
+  /* Let's build the expected filename of the requested plugin loader */
+  loader_basename = g_strdup_printf ("lib%sloader.%s", loader_id, G_MODULE_SUFFIX);
+
+  g_debug ("Loading loader '%s': '%s/%s'", loader_id, loader_dirname, loader_basename);
+
+  module = peas_object_module_new (loader_basename, loader_dirname, TRUE);
+
+  g_free (loader_basename);
+  g_free (loader_dirname);
+
+  if (!g_type_module_use (G_TYPE_MODULE (module)))
+    {
+      g_object_unref (module);
+      module = NULL;
+    }
+
+  return module;
+}
+
 static LoaderInfo *
 load_plugin_loader (PeasEngine  *engine,
                     const gchar *loader_id)
 {
-  gchar *loader_dirname;
-  gchar *loader_basename;
   guint i;
+  gchar *lc_loader_id;
   PeasObjectModule *module;
   PeasPluginLoader *loader;
 
-  /* Let's build the expected filename of the requested plugin loader */
-  loader_dirname = peas_dirs_get_plugin_loaders_dir ();
-  loader_basename = g_strdup_printf ("lib%sloader.%s", loader_id, G_MODULE_SUFFIX);
-  for (i = 0; loader_basename[i] != '\0'; ++i)
-    loader_basename[i] = g_ascii_tolower (loader_basename[i]);
+  /* We need to ensure we use the lowercase loader_id */
+  lc_loader_id = g_strdup (loader_id);
+  for (i = 0; lc_loader_id[i] != '\0'; ++i)
+    lc_loader_id[i] = g_ascii_tolower (lc_loader_id[i]);
 
-  g_debug ("Loading loader '%s': '%s/%s'", loader_id, loader_dirname, loader_basename);
+  module = try_to_open_loader_module (lc_loader_id, FALSE);
+  if (module == NULL)
+    module = try_to_open_loader_module (lc_loader_id, TRUE);
 
-  /* For now all modules are resident */
-  module = peas_object_module_new (loader_basename,
-                                   loader_dirname,
-                                   TRUE);
+  g_free (lc_loader_id);
 
-  g_free (loader_dirname);
-  g_free (loader_basename);
-
-  /* make sure to load the type definition */
-  if (g_type_module_use (G_TYPE_MODULE (module)))
+  if (module == NULL)
     {
-      peas_object_module_register_types (module);
-      loader = (PeasPluginLoader *) peas_object_module_create_object (module, PEAS_TYPE_PLUGIN_LOADER, 0, NULL);
-
-      if (loader == NULL || !PEAS_IS_PLUGIN_LOADER (loader))
-        {
-          g_warning ("Loader '%s' is not a valid PeasPluginLoader instance",
-                     loader_id);
-          if (loader != NULL && G_IS_OBJECT (loader))
-            g_object_unref (loader);
-          module = NULL;
-          loader = NULL;
-        }
-      else
-        {
-          gchar *module_dir = g_build_filename (engine->priv->base_module_dir, loader_id, NULL);
-          peas_plugin_loader_add_module_directory (loader, module_dir);
-          g_free (module_dir);
-        }
-
-      g_type_module_unuse (G_TYPE_MODULE (module));
+      g_warning ("Loader '%s' could not be loaded", loader_id);
+      return add_loader (engine, loader_id, NULL, NULL);
     }
-  else
+
+  loader = (PeasPluginLoader *) peas_object_module_create_object (module, PEAS_TYPE_PLUGIN_LOADER, 0, NULL);
+
+  if (loader == NULL || !PEAS_IS_PLUGIN_LOADER (loader))
     {
-      g_warning ("Loader '%s' could not be loaded",
-                 loader_id);
-      g_object_unref (module);
+      g_warning ("Loader '%s' is not a valid PeasPluginLoader instance", loader_id);
+      if (loader != NULL && G_IS_OBJECT (loader))
+        g_object_unref (loader);
       module = NULL;
       loader = NULL;
     }
 
+  g_type_module_unuse (G_TYPE_MODULE (module));
+
   return add_loader (engine, loader_id, module, loader);
 }
 
@@ -667,6 +725,9 @@ static gboolean
 load_plugin (PeasEngine     *engine,
              PeasPluginInfo *info)
 {
+  const gchar **dependencies;
+  PeasPluginInfo *dep_info;
+  guint i;
   PeasPluginLoader *loader;
 
   if (peas_plugin_info_is_loaded (info))
@@ -675,6 +736,24 @@ load_plugin (PeasEngine     *engine,
   if (!peas_plugin_info_is_available (info))
     return FALSE;
 
+  /* We set the plugin info as loaded before trying to load the dependencies,
+   * to make sure we won't have an infinite loop. */
+  info->loaded = TRUE;
+
+  dependencies = peas_plugin_info_get_dependencies (info);
+  for (i = 0; dependencies[i] != NULL; i++)
+    {
+      dep_info = peas_engine_get_plugin_info (engine, dependencies[i]);
+      if (!dep_info)
+        {
+          g_warning ("Plugin not found: %s", dependencies[i]);
+          return FALSE;
+        }
+
+      if (!peas_engine_load_plugin (engine, dep_info))
+        return FALSE;
+    }
+
   loader = get_plugin_loader (engine, info);
 
   if (loader == NULL)
@@ -685,11 +764,10 @@ load_plugin (PeasEngine     *engine,
       return FALSE;
     }
 
-  info->loaded = peas_plugin_loader_load (loader, info);
-
-  if (info->loaded == FALSE)
+  if (!peas_plugin_loader_load (loader, info))
     {
       g_warning ("Error loading plugin '%s'", info->name);
+      info->loaded = FALSE;
       info->available = FALSE;
       return FALSE;
     }
@@ -701,7 +779,8 @@ static void
 peas_engine_load_plugin_real (PeasEngine     *engine,
                               PeasPluginInfo *info)
 {
-  load_plugin (engine, info);
+  if (load_plugin (engine, info))
+    g_object_notify (G_OBJECT (engine), "loaded-plugins");
 }
 
 /**
@@ -710,8 +789,10 @@ peas_engine_load_plugin_real (PeasEngine     *engine,
  * @info: A #PeasPluginInfo.
  *
  * Loads the plugin corresponding to @info if it's not currently loaded.
+ * Emits the "load-plugin" signal; loading the plugin
+ * actually occurs in the default signal handler.
  *
- * Returns: whether the plugin has been successfuly loaded.
+ * Returns: whether the plugin has been successfully loaded.
  */
 gboolean
 peas_engine_load_plugin (PeasEngine     *engine,
@@ -734,19 +815,38 @@ static void
 peas_engine_unload_plugin_real (PeasEngine     *engine,
                                 PeasPluginInfo *info)
 {
+  GList *item;
+  const gchar *module_name;
   PeasPluginLoader *loader;
 
   if (!peas_plugin_info_is_loaded (info) ||
       !peas_plugin_info_is_available (info))
     return;
 
+  /* We set the plugin info as unloaded before trying to unload the
+   * dependants, to make sure we won't have an infinite loop. */
+  info->loaded = FALSE;
+
+  /* First unload all the dependant plugins */
+  module_name = peas_plugin_info_get_module_name (info);
+  for (item = engine->priv->plugin_list; item; item = item->next)
+    {
+      PeasPluginInfo *other_info = PEAS_PLUGIN_INFO (item->data);
+
+      if (!peas_plugin_info_is_loaded (other_info))
+        continue;
+
+      if (peas_plugin_info_has_dependency (other_info, module_name))
+         peas_engine_unload_plugin (engine, other_info);
+    }
+
   /* find the loader and tell it to gc and unload the plugin */
   loader = get_plugin_loader (engine, info);
 
   peas_plugin_loader_garbage_collect (loader);
   peas_plugin_loader_unload (loader, info);
 
-  info->loaded = FALSE;
+  g_object_notify (G_OBJECT (engine), "loaded-plugins");
 }
 
 /**
@@ -755,8 +855,10 @@ peas_engine_unload_plugin_real (PeasEngine     *engine,
  * @info: A #PeasPluginInfo.
  *
  * Unloads the plugin corresponding to @info.
+ * Emits the "unload-plugin" signal; unloading the plugin
+ * actually occurs in the default signal handler.
  *
- * Returns: whether the plugin has been successfuly unloaded.
+ * Returns: whether the plugin has been successfully unloaded.
  */
 gboolean
 peas_engine_unload_plugin (PeasEngine     *engine,
@@ -789,10 +891,29 @@ peas_engine_provides_extension (PeasEngine     *engine,
   return peas_plugin_loader_provides_extension (loader, info, extension_type);
 }
 
+/**
+ * peas_engine_create_extensionv:
+ * @engine: A #PeasEngine.
+ * @info: A loaded #PeasPluginInfo.
+ * @extension_type: The implemented extension #GType.
+ * @n_parameters: the length of the @parameters array.
+ * @parameters: an array of #GParameter.
+ *
+ * If the plugin identified by @info implements the @extension_type interface,
+ * then this function will return a new instance of this implementation,
+ * wrapped in a new #PeasExtension instance. Otherwise, it will return %NULL.
+ *
+ * See peas_engine_create_extension() for more information.
+ *
+ * Returns: (transfer full): a new instance of #PeasExtension wrapping
+ * the @extension_type instance, or %NULL.
+ */
 PeasExtension *
-peas_engine_get_extension (PeasEngine     *engine,
-                           PeasPluginInfo *info,
-                           GType           extension_type)
+peas_engine_create_extensionv (PeasEngine     *engine,
+                               PeasPluginInfo *info,
+                               GType           extension_type,
+                               guint           n_parameters,
+                               GParameter     *parameters)
 {
   PeasPluginLoader *loader;
 
@@ -800,18 +921,115 @@ peas_engine_get_extension (PeasEngine     *engine,
   g_return_val_if_fail (info != NULL, NULL);
 
   loader = get_plugin_loader (engine, info);
-  return peas_plugin_loader_get_extension (loader, info, extension_type);
+  return peas_plugin_loader_create_extension (loader, info, extension_type,
+                                              n_parameters, parameters);
+}
+
+/**
+ * peas_engine_create_extension_valist:
+ * @engine: A #PeasEngine.
+ * @info: A loaded #PeasPluginInfo.
+ * @extension_type: The implemented extension #GType.
+ * @first_property: the name of the first property.
+ * @var_args: the value of the first property, followed optionally by more
+ *   name/value pairs, followed by %NULL.
+ *
+ * If the plugin identified by @info implements the @extension_type interface,
+ * then this function will return a new instance of this implementation,
+ * wrapped in a new #PeasExtension instance. Otherwise, it will return %NULL.
+ *
+ * See peas_engine_create_extension() for more information.
+ *
+ * Returns: (transfer full): a new instance of #PeasExtension wrapping
+ * the @extension_type instance, or %NULL.
+ */
+PeasExtension *
+peas_engine_create_extension_valist (PeasEngine     *engine,
+                                     PeasPluginInfo *info,
+                                     GType           extension_type,
+                                     const gchar    *first_property,
+                                     va_list         var_args)
+{
+  gpointer type_struct;
+  guint n_parameters;
+  GParameter *parameters;
+  PeasExtension *exten;
+
+  type_struct = _g_type_struct_ref (extension_type);
+
+  if (!_valist_to_parameter_list (extension_type, type_struct, first_property,
+                                  var_args, &parameters, &n_parameters))
+    {
+      g_return_val_if_reached (NULL);
+    }
+
+  exten = peas_engine_create_extensionv (engine, info, extension_type,
+                                         n_parameters, parameters);
+
+  while (n_parameters-- > 0)
+    g_value_unset (&parameters[n_parameters].value);
+  g_free (parameters);
+
+  _g_type_struct_unref (extension_type, type_struct);
+
+  return exten;
+}
+
+/**
+ * peas_engine_create_extension:
+ * @engine: A #PeasEngine.
+ * @info: A loaded #PeasPluginInfo.
+ * @extension_type: The implemented extension #GType.
+ * @first_property: the name of the first property.
+ * @Varargs: the value of the first property, followed optionally by more
+ *   name/value pairs, followed by %NULL.
+ *
+ * If the plugin identified by @info implements the @extension_type interface,
+ * then this function will return a new instance of this implementation,
+ * wrapped in a new #PeasExtension instance. Otherwise, it will return %NULL.
+ *
+ * When creating the new instance of the @extension_type subtype, the
+ * provided construct properties will be passed to the extension construction
+ * handler (exactly like if you had called g_object_new() yourself).
+ *
+ * The new extension instance produced by this function will always be
+ * returned wrapped in a #PeasExtension proxy, following the current libpeas
+ * principle of never giving you the actual object (also because it might as
+ * well *not* be an actual object).
+ *
+ * Returns: (transfer full): a new instance of #PeasExtension wrapping
+ * the @extension_type instance, or %NULL.
+ */
+PeasExtension *
+peas_engine_create_extension (PeasEngine     *engine,
+                              PeasPluginInfo *info,
+                              GType           extension_type,
+                              const gchar    *first_property,
+                              ...)
+{
+  va_list var_args;
+  PeasExtension *exten;
+
+  va_start (var_args, first_property);
+  exten = peas_engine_create_extension_valist (engine, info, extension_type,
+                                               first_property, var_args);
+  va_end (var_args);
+
+  return exten;
 }
 
 /**
  * peas_engine_get_loaded_plugins:
  * @engine: A #PeasEngine.
  *
- * Returns the list of the names of all the loaded plugins, or %NULL if there
- * is no plugin currently loaded. Please note that the returned array is a
- * newly allocated one: you will need to free it using g_strfreev().
+ * Returns the list of the names of all the loaded plugins, or an array
+ * containing a single %NULL element if there is no plugin currently loaded.
  *
- * Returns: A newly-allocated NULL-terminated array of strings, or %NULL.
+ * Please note that the returned array is a newly allocated one: you will need
+ * to free it using g_strfreev().
+ *
+ * Returns: (transfer full): A newly-allocated %NULL-terminated
+ * array of strings, or %NULL.
  */
 gchar **
 peas_engine_get_loaded_plugins (PeasEngine *engine)
@@ -883,27 +1101,19 @@ peas_engine_set_loaded_plugins (PeasEngine   *engine,
 }
 
 /**
- * peas_engine_new:
- * @app_name: (allow-none): The name of the application
- * @base_module_dir: The base directory for language modules
- * @search_paths: The paths where to look for plugins
+ * peas_engine_get_default:
  *
- * Returns a new #PeasEngine object.
- * See the properties description for more information about the parameters.
+ * Return the existing instance of #PeasEngine or a subclass of it.
+ * If no #PeasEngine subclass has been instantiated yet, the first call
+ * of this function will return a new instance of #PeasEngine.
  *
- * Returns: a newly created #PeasEngine object.
+ * Returns: (transfer none): the existing instance of #PeasEngine.
  */
 PeasEngine *
-peas_engine_new (const gchar  *app_name,
-                 const gchar  *base_module_dir,
-                 const gchar **search_paths)
+peas_engine_get_default (void)
 {
-  g_return_val_if_fail (base_module_dir != NULL, NULL);
-  g_return_val_if_fail (search_paths != NULL, NULL);
+  if (default_engine == NULL)
+    return PEAS_ENGINE (g_object_new (PEAS_TYPE_ENGINE, NULL));
 
-  return PEAS_ENGINE (g_object_new (PEAS_TYPE_ENGINE,
-                                    "app-name", app_name,
-                                    "base-module-dir", base_module_dir,
-                                    "search-paths", search_paths,
-                                    NULL));
+  return default_engine;
 }
