@@ -23,25 +23,24 @@
 #include <config.h>
 #endif
 
+#include <string.h>
+#include <gmodule.h>
+
 #include "peas-plugin-loader-c.h"
 #include "peas-extension-c.h"
 #include <libpeas/peas-object-module.h>
 #include <libpeas/peas-extension-base.h>
-#include <gmodule.h>
 
 struct _PeasPluginLoaderCPrivate
 {
   GHashTable *loaded_plugins;
 };
 
-G_DEFINE_DYNAMIC_TYPE (PeasPluginLoaderC, peas_plugin_loader_c, PEAS_TYPE_PLUGIN_LOADER);
+G_DEFINE_TYPE (PeasPluginLoaderC, peas_plugin_loader_c, PEAS_TYPE_PLUGIN_LOADER);
 
 G_MODULE_EXPORT void
 peas_register_types (PeasObjectModule *module)
 {
-  peas_plugin_loader_c_register_type (G_TYPE_MODULE (module));
-  peas_extension_c_register (G_TYPE_MODULE (module));
-
   peas_object_module_register_extension_type (module,
                                               PEAS_TYPE_PLUGIN_LOADER,
                                               PEAS_TYPE_PLUGIN_LOADER_C);
@@ -107,26 +106,41 @@ peas_plugin_loader_c_provides_extension  (PeasPluginLoader *loader,
 }
 
 static PeasExtension *
-peas_plugin_loader_c_get_extension (PeasPluginLoader *loader,
-                                    PeasPluginInfo   *info,
-                                    GType             exten_type)
+peas_plugin_loader_c_create_extension (PeasPluginLoader *loader,
+                                       PeasPluginInfo   *info,
+                                       GType             exten_type,
+                                       guint             n_parameters,
+                                       GParameter       *parameters)
 {
   PeasPluginLoaderC *cloader = PEAS_PLUGIN_LOADER_C (loader);
   PeasObjectModule *module;
-  GParameter info_param = { 0 };
+  GParameter *exten_parameters;
   gpointer instance;
 
   module = (PeasObjectModule *) g_hash_table_lookup (cloader->priv->loaded_plugins,
                                                      info);
   g_return_val_if_fail (module != NULL, NULL);
 
-  info_param.name = "plugin-info";
-  g_value_init (&info_param.value, PEAS_TYPE_PLUGIN_INFO);
-  g_value_set_boxed (&info_param.value, info);
+  /* We want to add a "plugin-info" property so we can pass it to the extension
+   * if it inherits from PeasExtensionBase. No need to actually "duplicate" the
+   * GValues, a memcpy is sufficient as the source GValues are longer lived
+   * than our local copy. */
+  exten_parameters = g_new (GParameter, n_parameters + 1);
+  memcpy (exten_parameters, parameters, sizeof (GParameter) * n_parameters);
 
-  instance = peas_object_module_create_object (module, exten_type, 1, &info_param);
+  /* Initialize our additional property */
+  exten_parameters[n_parameters].name = g_intern_static_string ("plugin-info");
+  memset (&exten_parameters[n_parameters].value, 0, sizeof (GValue));
+  g_value_init (&exten_parameters[n_parameters].value, PEAS_TYPE_PLUGIN_INFO);
+  g_value_set_boxed (&exten_parameters[n_parameters].value, info);
 
-  g_value_unset (&info_param.value);
+  instance = peas_object_module_create_object (module,
+                                               exten_type,
+                                               n_parameters + 1,
+                                               exten_parameters);
+
+  g_value_unset (&exten_parameters[n_parameters].value);
+  g_free (exten_parameters);
 
   if (instance == NULL)
     {
@@ -210,12 +224,7 @@ peas_plugin_loader_c_class_init (PeasPluginLoaderCClass *klass)
   loader_class->load = peas_plugin_loader_c_load;
   loader_class->unload = peas_plugin_loader_c_unload;
   loader_class->provides_extension = peas_plugin_loader_c_provides_extension;
-  loader_class->get_extension = peas_plugin_loader_c_get_extension;
+  loader_class->create_extension = peas_plugin_loader_c_create_extension;
 
   g_type_class_add_private (object_class, sizeof (PeasPluginLoaderCPrivate));
 }
-
-static void
-peas_plugin_loader_c_class_finalize (PeasPluginLoaderCClass *klass)
-{
-}
