@@ -30,15 +30,10 @@
 #include <Python.h>
 #include <pygobject.h>
 #include <libpeas/peas-introspection.h>
+#include <libpeas/peas-extension-subclasses.h>
 #include "peas-extension-python.h"
 
-G_DEFINE_DYNAMIC_TYPE (PeasExtensionPython, peas_extension_python, PEAS_TYPE_EXTENSION);
-
-void
-peas_extension_python_register (GTypeModule *module)
-{
-  peas_extension_python_register_type (module);
-}
+G_DEFINE_TYPE (PeasExtensionPython, peas_extension_python, PEAS_TYPE_EXTENSION);
 
 static void
 peas_extension_python_init (PeasExtensionPython *pyexten)
@@ -48,8 +43,8 @@ peas_extension_python_init (PeasExtensionPython *pyexten)
 static gboolean
 peas_extension_python_call (PeasExtension *exten,
                             const gchar   *method_name,
-                            GArgument     *args,
-                            GArgument     *retval)
+                            GIArgument    *args,
+                            GIArgument    *retval)
 {
   PeasExtensionPython *pyexten = PEAS_EXTENSION_PYTHON (exten);
   GType gtype;
@@ -85,18 +80,15 @@ peas_extension_python_class_init (PeasExtensionPythonClass *klass)
   extension_class->call = peas_extension_python_call;
 }
 
-static void
-peas_extension_python_class_finalize (PeasExtensionPythonClass *klass)
-{
-}
-
 PeasExtension *
 peas_extension_python_new (GType     gtype,
                            PyObject *instance)
 {
   PeasExtensionPython *pyexten;
+  GType real_type;
 
-  pyexten = PEAS_EXTENSION_PYTHON (g_object_new (PEAS_TYPE_EXTENSION_PYTHON,
+  real_type = peas_extension_register_subclass (PEAS_TYPE_EXTENSION_PYTHON, gtype);
+  pyexten = PEAS_EXTENSION_PYTHON (g_object_new (real_type,
                                                  "extension-type", gtype,
                                                  NULL));
   pyexten->instance = instance;
