@@ -57,14 +57,11 @@ static PyObject *PyGObject_Type;
 static gboolean   peas_plugin_loader_python_add_module_path (PeasPluginLoaderPython *self,
                                                              const gchar            *module_path);
 
-G_DEFINE_DYNAMIC_TYPE (PeasPluginLoaderPython, peas_plugin_loader_python, PEAS_TYPE_PLUGIN_LOADER);
+G_DEFINE_TYPE (PeasPluginLoaderPython, peas_plugin_loader_python, PEAS_TYPE_PLUGIN_LOADER);
 
 G_MODULE_EXPORT void
 peas_register_types (PeasObjectModule *module)
 {
-  peas_plugin_loader_python_register_type (G_TYPE_MODULE (module));
-  peas_extension_python_register (G_TYPE_MODULE (module));
-
   peas_object_module_register_extension_type (module,
                                               PEAS_TYPE_PLUGIN_LOADER,
                                               PEAS_TYPE_PLUGIN_LOADER_PYTHON);
@@ -102,6 +99,7 @@ find_python_extension_type (PeasPluginInfo *info,
         case 0:
           continue;
         case -1:
+        default:
           PyErr_Print ();
           continue;
         }
@@ -136,9 +134,11 @@ peas_plugin_loader_python_provides_extension (PeasPluginLoader *loader,
 }
 
 static PeasExtension *
-peas_plugin_loader_python_get_extension (PeasPluginLoader *loader,
-                                         PeasPluginInfo   *info,
-                                         GType             exten_type)
+peas_plugin_loader_python_create_extension (PeasPluginLoader *loader,
+                                            PeasPluginInfo   *info,
+                                            GType             exten_type,
+                                            guint             n_parameters,
+                                            GParameter       *parameters)
 {
   PeasPluginLoaderPython *pyloader = PEAS_PLUGIN_LOADER_PYTHON (loader);
   PythonInfo *pyinfo;
@@ -190,7 +190,7 @@ peas_plugin_loader_python_get_extension (PeasPluginLoader *loader,
       return NULL;
     }
 
-  pygobject_construct (pygobject, NULL);
+  pygobject_constructv (pygobject, n_parameters, parameters);
 
   if (pygobject->obj == NULL)
     {
@@ -416,7 +416,7 @@ peas_plugin_loader_python_add_module_path (PeasPluginLoaderPython *self,
   if (!Py_IsInitialized ())
     return FALSE;
 
-  pathlist = PySys_GetObject ("path");
+  pathlist = PySys_GetObject ((char*) "path");
   pathstring = PyString_FromString (module_path);
 
   if (PySequence_Contains (pathlist, pathstring) == 0)
@@ -430,7 +430,7 @@ static gboolean
 peas_python_init (PeasPluginLoaderPython *loader)
 {
   PyObject *mdict, *gobject, *gi, *gettext, *install, *gettext_args;
-  char *argv[] = { "", NULL };
+  const char *argv[] = { "", NULL };
   gchar *prgname;
 
   /* We are trying to initialize Python for the first time,
@@ -450,10 +450,10 @@ peas_python_init (PeasPluginLoaderPython *loader)
     argv[0] = prgname;
 
 #if PY_VERSION_HEX < 0x02060600
-  PySys_SetArgv (1, argv);
+  PySys_SetArgv (1, (char**) argv);
   PyRun_SimpleString ("import sys; sys.path.pop(0)\n");
 #else
-  PySys_SetArgvEx (1, argv, 0);
+  PySys_SetArgvEx (1, (char**) argv, 0);
 #endif
 
   /* Note that we don't call this with the GIL held, since we haven't initialised pygobject yet */
@@ -584,14 +584,9 @@ peas_plugin_loader_python_class_init (PeasPluginLoaderPythonClass *klass)
   loader_class->add_module_directory = peas_plugin_loader_python_add_module_directory;
   loader_class->load = peas_plugin_loader_python_load;
   loader_class->unload = peas_plugin_loader_python_unload;
-  loader_class->get_extension = peas_plugin_loader_python_get_extension;
+  loader_class->create_extension = peas_plugin_loader_python_create_extension;
   loader_class->provides_extension = peas_plugin_loader_python_provides_extension;
   loader_class->garbage_collect = peas_plugin_loader_python_garbage_collect;
 
   g_type_class_add_private (object_class, sizeof (PeasPluginLoaderPythonPrivate));
 }
-
-static void
-peas_plugin_loader_python_class_finalize (PeasPluginLoaderPythonClass *klass)
-{
-}
